// ------------------------------------------------------------
// Biblioteka obslugujaca LCD
// ------------------------------------------------------------

#ifndef _LCD_LIB_H_
#define _LCD_LIB_H_

#include <util/delay.h>
#include <avr/pgmspace.h>
#include "portpins_structs.h"

// ------------------------------------------------------------
// Konfiguracja LCD
// ------------------------------------------------------------
// simple protocol - 4 Data lines, RegisterSelect and Enable
// pin setup
#ifndef LCD_RS_PPS
#error "struct LCD_RS_PPS not defined !"
#endif
#ifndef LCD_E_PPS
#error "struct LCD_E_PPS not defined !"
#endif
#ifndef LCD_D4_PPS
#error "struct LCD_D4_PPS not defined !"
#endif
#ifndef LCD_D5_PPS
#error "struct LCD_D5_PPS not defined !"
#endif
#ifndef LCD_D6_PPS
#error "struct LCD_D6_PPS not defined !"
#endif
#ifndef LCD_D7_PPS
#error "struct LCD_D7_PPS not defined !"
#endif

// przydatne stae
#define LCD_GOTO_1LINE    0x00
#define LCD_GOTO_2LINE    0x40

#define LCD_DISPLAY_CONTROL		0x08
#define LCD_DC_DISPLAY_ON		0x04
#define LCD_DC_CURSOR_ON		0x02
#define LCD_DC_BLINKING_ON		0x01
// ---------------------------------------------------------------
// Stale
// ---------------------------------------------------------------
#define LCD_POWER_ON_TIME    40 // 40 ms po wlaczeniu zasilania
#define LCD_WRITE_TIME       40 // 40 us po wyslaniu kazdego bajtu
#define LCD_INIT1_TIME     4100 // 4,1 ms po pierwszym 0x3*
#define LCD_INIT2_TIME      100 // 100 us po kolejnych 0x3*
#define LCD_CLEAN_TIME     1600 // 1,64 ms (dodatkowe 40 us opoznienie w LCD_write)
#define LCD_CLEAN          0b00000001 // czyszczenie wyswietlacza
#define LCD_RETURN         0b00000010 // powrot kursora
#define LCD_ENTRY_MODE     0b00000110 // adres zwiekszany, przesuwany kursor
#define LCD_ON_OFF_MODE    0b00001100 // wyswietlanie wlaczone, bez kursora
#define LCD_SETUP          0b00101000 // interfejs 4-bitowy, wyswietlacz 2-liniowy, matryca 5x8 punktow
#define LCD_DDRAM_ADDR     0x80   // ustaw adres pamieci danych


// ------------------------------------------------------------
// Inicjalizacja LCD
// ------------------------------------------------------------

static void LCD_init ()
{
	uint8_t i;
	// Skonfiguruj linie jako wyjscia
	OUTPUT(LCD_RS_PPS);
	OUTPUT(LCD_E_PPS);
	OUTPUT(LCD_D4_PPS);
	OUTPUT(LCD_D5_PPS);
	OUTPUT(LCD_D6_PPS);
	OUTPUT(LCD_D7_PPS);

	// Bedziemy wysylac instrukcje
    LOW(LCD_RS_PPS);
	LOW(LCD_E_PPS);
	
	// Poczekaj po wlaczeniu zasilania
	_delay_ms(LCD_POWER_ON_TIME);

	// Wyslij pierwsze 0x3* i poczekaj
    HIGH(LCD_E_PPS);//    sbi     PORT_LCD_C, LCD_E
    HIGH(LCD_D4_PPS);//    sbi     PORT_LCD_D, LCD_D4
    HIGH(LCD_D5_PPS);//    sbi     PORT_LCD_D, LCD_D5
    LOW(LCD_D6_PPS);//    cbi     PORT_LCD_D, LCD_D6
    LOW(LCD_D7_PPS);//    cbi     PORT_LCD_D, LCD_D7
	_delay_us(1);
    LOW(LCD_E_PPS);//    cbi     PORT_LCD_C, LCD_E
    _delay_us(LCD_INIT1_TIME);//    rcall   delay_us

	// Dwukrotnie wyslij 0x3*
	for (i = 0; i < 2; i++)
	{
		HIGH(LCD_E_PPS);
		_delay_us(1);
		LOW(LCD_E_PPS);
		_delay_us(LCD_INIT2_TIME);
	}

	// Przelacz w tryb 4-bitowy
	HIGH(LCD_E_PPS);//        sbi     PORT_LCD_C, LCD_E
    LOW(LCD_D4_PPS);//    cbi     PORT_LCD_D, LCD_D4
    _delay_us(1);
	LOW(LCD_E_PPS);
	_delay_us(LCD_WRITE_TIME);
	
}
// ------------------------------------------------------------
// Konfiguracja LCD
// ------------------------------------------------------------
static void LCD_command (uint8_t data);
static void LCD_config ()
{
	LCD_command (LCD_SETUP);
	LCD_command (LCD_ENTRY_MODE);
	LCD_command (LCD_ON_OFF_MODE);
}

// ------------------------------------------------------------
// Zapis r16 do LCD
// ------------------------------------------------------------
static void LCD_write(uint8_t data)
{
	uint8_t i;
	for (i = 0; i < 2; i++)
	{
		HIGH(LCD_E_PPS);
		LOW(LCD_D4_PPS);
		LOW(LCD_D5_PPS);
		LOW(LCD_D6_PPS);
		LOW(LCD_D7_PPS);
		if (0x10 & data) HIGH(LCD_D4_PPS);
		if (0x20 & data) HIGH(LCD_D5_PPS);
		if (0x40 & data) HIGH(LCD_D6_PPS);
		if (0x80 & data) HIGH(LCD_D7_PPS);
		_delay_us(1);
		LOW(LCD_E_PPS);
		
		data <<= 4;
	}
	_delay_us(LCD_WRITE_TIME);
}
	
// ------------------------------------------------------------
// Wyslanie znaku z r16 do LCD
// ------------------------------------------------------------
static void LCD_data (uint8_t data)
{
	HIGH(LCD_RS_PPS);
	LCD_write(data);
}

// ------------------------------------------------------------
// Wyslanie polecenia z r16 do LCD
// ------------------------------------------------------------
static void LCD_command (uint8_t data)
{
	LOW(LCD_RS_PPS);
	LCD_write(data);
}

// ------------------------------------------------------------
// Wyswietlenie dziesietnie liczby z r16 na LCD
//  r16 jest niszczony!
// ------------------------------------------------------------
#define GET_DECIMAL(number,mask,result)\
do {\
	result = 0;\
	while (number >= mask)\
	{\
		number -= mask;\
		result++;\
	}\
} while(0)


static void LCD_number (uint8_t number)
{
	uint8_t digit, zero;
	GET_DECIMAL(number, 100, digit);
	zero = digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 10, digit);
	zero |= digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 1, digit);
	LCD_data(digit + '0');
}

static void LCD_number16 (uint16_t number)
{
	uint8_t digit, zero;
	GET_DECIMAL(number, 10000, digit);
	zero = digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 1000, digit);
	zero |= digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 100, digit);
	zero |= digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 10, digit);
	zero |= digit;
	if (zero)
	{
		LCD_data(digit + '0');	
	}
	else
	{
		LCD_data(' ');
	}
	GET_DECIMAL(number, 1, digit);
	LCD_data(digit + '0');
}


static void LCD_nibble (uint8_t nibble)
{
	nibble += '0';
	if (nibble > '9')
	{
		nibble += 'A' - '0' - 10;
	}
	LCD_data(nibble);
}
static void LCD_hex (uint8_t number)
{
	uint8_t nibble;
	nibble = number >> 4;
	LCD_nibble (nibble);
	nibble = 0x0F & number;
	LCD_nibble (nibble);
}
// ------------------------------------------------------------
// Wyswietlenie tekstu
//  z   - adres poczatku tekstu
//  r16 - pozycja pierwszego znaku na wyswietlaczu
//  r16 i z sa niszczone!
// ------------------------------------------------------------
static void LCD_write_text (const char * text)
{
	while (*text)
	{
		LCD_data(*text++);
	}
	
}
// wypisuje tekst umieszczony w pamici flash
static void LCD_write_text_P (const prog_char * text)
{
	char c = pgm_read_byte (text++);
	while (c)
	{
		LCD_data(c);
		c = pgm_read_byte (text++);
	}
	
}

// --------------------------------------------------------------
// Ustawienie wyswietlania na pozycji z r16
// --------------------------------------------------------------
static void LCD_goto (uint8_t address)
{
	LCD_command(LCD_DDRAM_ADDR | address);
}

// ---------------------------------------------------------------
// Czyszczenie wyswietlacza
// ---------------------------------------------------------------
static void LCD_clear (void)
{
	LCD_command(LCD_CLEAN);
	_delay_us(LCD_CLEAN_TIME);
}




#endif /* _LCD_LIB_H_ */ 
