#ifndef _PORTPINS_STRUCTS_H_
#define _PORTPINS_STRUCTS_H_

/* this file allow using port as one structure, eg:
 *
 * #define LED_PS PB2S
 * DDR(LED_PS) = (1 << P(LED_PS));
 * DDR(LED_PS) = PMASK(LED_PS);
 * PORT(LED_PS) = 0xaa;
 * uint8_t input = PIN(LED_PS);
 */

#include <avr/io.h>

#define PORT(x) (*(volatile uint8_t *)((x >> 24) & 0xff))
#define DDR(x)  (*(volatile uint8_t *)((x >> 16) & 0xff))
#define PIN(x)  (*(volatile uint8_t *)((x >>  8) & 0xff))
#define P(x)    ((uint8_t)(x & 0xff))
#define PMASK(x) (1 << P(x))

#define INPUT(x)  DDR(x)&=~PMASK(x)
#define OUTPUT(x) DDR(x)|=PMASK(x)
#define HIGH(x) PORT(x)|=PMASK(x)
#define LOW(x)  PORT(x)&=~PMASK(x)
#define TOGGLE(x) PORT(x)^=PMASK(x)
#define READ(x) (PIN(x)&PMASK(x))

/* PORTA */
#define PA0S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA0))
#define PA1S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA1))
#define PA2S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA2))
#define PA3S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA3))
#define PA4S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA4))
#define PA5S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA5))
#define PA6S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA6))
#define PA7S (((uint32_t)(intptr_t)(&PORTA) << 24) | ((uint32_t)(intptr_t)(&DDRA) << 16) | ((uint32_t)(intptr_t)(&PINA) << 8) | (PA7))

/* PORTB */
#define PB0S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB0))
#define PB1S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB1))
#define PB2S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB2))
#define PB3S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB3))
#define PB4S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB4))
#define PB5S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB5))
#define PB6S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB6))
#define PB7S (((uint32_t)(intptr_t)(&PORTB) << 24) | ((uint32_t)(intptr_t)(&DDRB) << 16) | ((uint32_t)(intptr_t)(&PINB) << 8) | (PB7))

/* PORTC */
#define PC0S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC0))
#define PC1S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC1))
#define PC2S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC2))
#define PC3S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC3))
#define PC4S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC4))
#define PC5S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC5))
#define PC6S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC6))
#define PC7S (((uint32_t)(intptr_t)(&PORTC) << 24) | ((uint32_t)(intptr_t)(&DDRC) << 16) | ((uint32_t)(intptr_t)(&PINC) << 8) | (PC7))

/* PORTD */
#define PD0S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD0))
#define PD1S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD1))
#define PD2S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD2))
#define PD3S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD3))
#define PD4S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD4))
#define PD5S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD5))
#define PD6S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD6))
#define PD7S (((uint32_t)(intptr_t)(&PORTD) << 24) | ((uint32_t)(intptr_t)(&DDRD) << 16) | ((uint32_t)(intptr_t)(&PIND) << 8) | (PD7))

/* PORTE */
#define PE0S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE0))
#define PE1S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE1))
#define PE2S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE2))
#define PE3S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE3))
#define PE4S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE4))
#define PE5S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE5))
#define PE6S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE6))
#define PE7S (((uint32_t)(intptr_t)(&PORTE) << 24) | ((uint32_t)(intptr_t)(&DDRE) << 16) | ((uint32_t)(intptr_t)(&PINE) << 8) | (PE7))



#endif /* _PORTPINS_STRUCTS_H_ */